**************************************************************************************
SAS code used to demonstrate how to use the SASphcpe.exe software in SAS;            *
SASphcpe.exe is used to compute Gonen and Heller Concordance probability.            *
Concordance probability and discriminatory power in proportional hazards regression  *
(Biometrika (2005), 92, 4, pp. 965-970;                                              *
**************************************************************************************

If you find any bugs, please contact Quincy Mo (moq@mskcc.org) at the 
Department of Epidemiology and Biostatistics
Memorial Sloan-Kettering Cancer Center, New York
**************************************************************************************;

/*
Definition of macro variables:
DSN:    name of the SAS data set containing survival times, status and covariates
SURV:   survival time
EVENT:  status indicator (1: event, 0: censored)
COVS:   list of covariates, seperated by blanks. 
EXELOC: location where the executable file SASphcpe.exe is saved.
OUTLOC: location where intermediate files will be stored. Any valid path is allowed. 
        defaults to the value of EXELOC
*/

%macro cpe(dsn=,surv=,event=,covs=,exeloc=,outloc=&exeloc);

proc phreg data=&dsn outest=_est1 covout noprint;
  model &surv*&event(0)=&covs;
  output out=_q1 xbeta=xbeta;
run;

data _parameter(keep=&covs);
  set _est1(where=(_type_='PARMS'));
run;

data _varbeta(keep=&covs);
  set _est1(where=(_type_='COV'));
run;

data _design;
  set _q1(keep=&covs);
  if nmiss(of &covs)>0 then delete;
run;

data _phcpedata;
set _parameter _varbeta _design;
run;

*SASphcpe.exe can only correctly read tab or space delimited text file;
proc export data=_phcpedata outfile="&outloc.\_phcpedata_.txt" 
	dbms=dlm replace; 
	delimiter = " ";
 * delimiter = '09'x;
run;

X "&exeloc.\SASphcpe.exe &outloc.\_phcpedata_.txt &outloc.\_phcpeout_.txt";

data phcpeResult;
infile "&outloc.\_phcpeout_.txt" dlm = '09'x firstobs=2;
input CPE StdErrorCPE;
run;

proc print data=phcpeResult; run;

%mend;


